﻿using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.BusinessLogicsContracts;
using BlacksmithWorkshopContracts.SearchModels;
using BlacksmithWorkshopContracts.StoragesContracts;
using BlacksmithWorkshopContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BlacksmithWorkshopBusinessLogic.BusinessLogic
{
    internal class ArticleLogic : IArticleLogic
    {
        private readonly ILogger _logger;

        private readonly IArticleStorage _articleStorage;

        //конструктор
        public ArticleLogic(ILogger<ArticleLogic> logger, IArticleStorage productStorage)
        {
            _logger = logger;
            _articleStorage = productStorage;
        }

        //вывод отфильтрованного списка
        public List<ArticleViewModel>? ReadList(ArticleSearchModel? model)
        {
            _logger.LogInformation("ReadList. ArticleName:{ArticleName}. Id:{Id}", model?.ArticleName, model?.Id);

            //list хранит весь список в случае, если model пришло со значением null на вход метода
            var list = model == null ? _articleStorage.GetFullList() : _articleStorage.GetFilteredList(model);

            if(list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }

            _logger.LogInformation("ReadList. Count:{Count}", list.Count);

            return list;
        }

        //вывод конкретного изделия
        public ArticleViewModel? ReadProduct(ArticleSearchModel model)
        {
            if(model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            _logger.LogInformation("ReadProduct. ArticleName:{ArticleName}. Id:{Id}", model.ArticleName, model.Id);

            var element = _articleStorage.GetElement(model);

            if(element == null)
            {
                _logger.LogWarning("ReadProduct element not found");
                return null;
            }

            _logger.LogInformation("ReadProduct find. Id:{Id}", model.Id);

            return element;
        }

        //Создание изделия
        public bool Create(ArticleBindingModel model)
        {
            CheckModel(model);

            if(_articleStorage.Insert(model) == null)
            {
                _logger.LogWarning("Create operation failed");
                return false;
            }

            return true;
        }

        //обновление изделия
        public bool Update(ArticleBindingModel model)
        {
            CheckModel(model);

            if(_articleStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }

            return true;
        }

        //удаление изделия
        public bool Delete(ArticleBindingModel model)
        {
            CheckModel(model);

            if(_articleStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }

            return true;
        }

        //проверка входного аргумента для методов Insert, Update и Delete
        private void CheckModel(ArticleBindingModel model, bool withParams = true)
        {
            if(model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            //если без параметров?
            if (!withParams)
            {
                return;
            }

            //проверка на наличие названия изделия
            if (string.IsNullOrEmpty(model.ArticleName))
            {
                throw new ArgumentNullException("Нет названия изделия", nameof(model.ArticleName));
            }

            //проверка на наличие нормальной цены у изделия
            if(model.Price <= 0)
            {
                throw new ArgumentNullException("Цена изделия должна быть больше 0", nameof(model.Price));
            }

            _logger.LogInformation("Article. ArticleName:{ArticleName}. Price:{Price}. Id:{Id}", 
                model.ArticleName, model.Price, model.Id);

            //проверка на наличие такого же изделия в списке
            var element = _articleStorage.GetElement(new ArticleSearchModel
            {
                ArticleName = model.ArticleName,
            });

            if(element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Изделие с таким названием уже есть");
            }
        }
    }
}
